﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
/* Classes For Making Web Requests and Processing Responses */
using System.IO;
using System.Net;
using System.Net.Http;

namespace touchit
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            /* Insert basic information pertaining to the user using the application
             * If a user just runs the application (double-clicks it) the useranme and domain are available via Environment.UserName and Environment.UserDomainName.
             * If the user chooses to "Run as different user" the Environment information reflects the "run as" user.
             * If System.Security.Principal.WindowsIdentity.GetCurrent().Name is used, that creates output such as domain\username...this is sort of old school.
             */
             if (Environment.UserName.ToString().Length > 0) { userNameBox.Text = Environment.UserName.ToString(); }
             if (Environment.UserDomainName.ToString().Length > 0) { domainBox.Text = Environment.UserDomainName.ToString(); }
        }

        private void submitBtn_Click(object sender, EventArgs e)
        {
            /* Clear UI text areas */
            responseCookieBox.Text = "";
            responseHeadersBox.Text = "";
            responseFullHeadersBox.Text = "";
            responseDataBox.Text = "";

            /* Get data to pass to touchit() */
            var tURL = targetURLBox.Text;
            var tURI = targetURIBox.Text;
            var sUser = userNameBox.Text;
            var sPass = passwordBox.Text; /* Note: In the properties of the passwordBox the smiley face as the passwordchar is entered by pressing Alt + 1 on the keypad */
            var sDomain = domainBox.Text;
            var sHandleCreds = (selAuthenticationBox.Text == "yes") ? true : false;

            /*
             * Call touchit() Task and wait synchronously for completion.
             * "touchit().Result" ensures a synchronous behavior where execution pauses at this line until touchit() is done and returns the string value into Task<>.
             * "await touchit()" would not stop execution at this line until touchit() was done.
             */
            var touchResult = touchit(tURL, tURI, sUser, sPass, sDomain, sHandleCreds).Result;

            /* Go through the list of results from touchit() in the form of List<KeyValuePair<string, string>> */
            var responseStatusCode = "";
            var responseCookie = "";
            var responseHeaders = "";
            var responseFullHeaders = "";
            var responseData = "";
            foreach (KeyValuePair<string, string> keyValuePairs in touchResult)
            {
                if (keyValuePairs.Key.ToLower() == "statuscode") { responseStatusCode = keyValuePairs.Value; }
                if (keyValuePairs.Key.ToLower() == "cookie") { responseCookie = keyValuePairs.Value; }
                if (keyValuePairs.Key.ToLower() == "headers") { responseHeaders = keyValuePairs.Value; }
                if (keyValuePairs.Key.ToLower() == "fullheaders") { responseFullHeaders = keyValuePairs.Value; }
                if (keyValuePairs.Key.ToLower() == "data") { responseData = keyValuePairs.Value; }
            }

            /* Dump response results to UI */
            statusCodeLabel.Text = responseStatusCode;
            responseCookieBox.Text = responseCookie;
            responseHeadersBox.Text = responseHeaders;
            responseFullHeadersBox.Text = responseFullHeaders;
            responseDataBox.Text = responseData;
        }

        private async Task<List<KeyValuePair<string, string>>> touchit(string tURL, string tURI, string sUser, string sPass, string sDomain, bool sHandleCreds)
        {
            /*
             * ARGUMENTS FOR touchit(tURL, tURI, sUser, sPass, sDomain, sHandleCreds)
             * tURL = Example: https://www.somesite.com/
             * tURI = Example: /folder/file.asmx
             * sUser = userName for NTLM authentication
             * sPass = userPassword for NTLM authentication
             * sDomain = network-domain for NTLM authentication
             * sHandleCreds = true/false (true = pass local credentials when no NTLM is present, false = do not pass local credentials when no NTLM is present)
             */
            /*
             * RESPONSE FOR touchit() is List<KeyValuePair<string, string>> in the form of:
             * statuscode = status code such as OK for 200
             * cookie = name = value pairs for cookies separated by Environment.NewLine
             * headers = basic response header information such as Content-Length and Content-Type
             * fullheaders = the entire response header
             * data = the body of the response; for example, the source code of a webpage
             */
            var targetURL = tURL;
            var targetURI = tURI;
            var userName = sUser;
            var password = sPass;
            var domain = sDomain;
            var useDefaultCredsCheckbox = sHandleCreds;

            /* Define http and cookie classes to utilize */
            CookieContainer cookies = new CookieContainer();
            HttpClientHandler handler = new HttpClientHandler();
            handler.CookieContainer = cookies;
            handler.UseDefaultCredentials = false;

            /* Handle credentials of the user */
            var cache = new CredentialCache();

            /* Determine if credentials should be passed with request or not */
            if (userName.Length > 0 && password.Length > 0 && domain.Length > 0)
            {
                /* User the user-specified NTLM credentials */
                var credential = new NetworkCredential(userName, password, domain);
                cache.Add(new Uri(targetURL), "NTLM", credential);
                handler.Credentials = credential;
                handler.UseDefaultCredentials = false;
            }
            else
            {
                /* Determine if default credentials should be passed with the request or not.  If this is false and no NTLM are specified (above), no credentials are passed with request. */
                if (useDefaultCredsCheckbox == true)
                {
                    /* Pass the current default credentials the user is using on the computer for the application that is running */
                    handler.UseDefaultCredentials = true;
                }
            }

            /* Make http request and get response */
            HttpClient client = new HttpClient(handler);
            client.BaseAddress = new Uri(targetURL);
            HttpResponseMessage response = client.GetAsync(targetURL + targetURI).Result;
            var responseStatusCode = response.StatusCode.ToString(); /* This will contain OK (OK for 200, Unauthorized for 401, NotFound for 404, etc) */
            var responseFullHeaders = response.ToString(); /* Contains the entire response header */
            var responseHeaders = response.Content.Headers.ToString(); /* Contains just basic response header information such as content-lengh, content-type, last-modified */

            /* Get the actual webpage source code */
            Stream responseStream = await response.Content.ReadAsStreamAsync();
            var responseData = "";
            using (StreamReader responseReader = new StreamReader(responseStream))
            {
                responseData = responseReader.ReadToEnd();
            }

            /* Get cookies */
            IEnumerable<Cookie> responseCookies = cookies.GetCookies(client.BaseAddress).Cast<Cookie>();
            var responseCookie = "";
            foreach (Cookie cookie in responseCookies)
            {
                responseCookie += cookie.Name.ToString() + " = " + cookie.Value.ToString() + Environment.NewLine;
            }

            /* Package response data so it can be returned */
            var responseList = new List<KeyValuePair<string, string>>()
            {
                new KeyValuePair<string, string>("statuscode", responseStatusCode),
                new KeyValuePair<string, string>("cookie", responseCookie),
                new KeyValuePair<string, string>("headers", responseHeaders),
                new KeyValuePair<string, string>("fullheaders", responseFullHeaders),
                new KeyValuePair<string, string>("data", responseData)
            };
            
            return responseList;
        }
    }
}
